require 'active_model'
require 'json'
require 'set'

module Xpost
  class Order
    include ActiveModel::Model

    CONFIRMABLE_PARAMETERS  = Set[:status, :email, :buyer_name, :contact_number, :delivery_address, :items,
                                  :shipment, :total, :currency, :payment_method, :payment_provider]
    FOR_PICKUP_PARAMETERS   = Set[:pickup_address, :pickup_at]

    attr_accessor :email, :buyer_name, :contact_number, :reference_id
    attr_accessor :total, :currency, :payment_method, :payment_provider
    attr_accessor :delivery_address, :pickup_address, :pickup_at
    attr_accessor :status, :items, :shipment

    validates_presence_of :email, :buyer_name, :contact_number
    validates_presence_of :delivery_address

    def initialize(options = {})
      @status           = options[:status]            || "pending"
      @reference_id     = options[:reference_id]
      @email            = options[:email]
      @buyer_name       = options[:buyer_name]
      @contact_number   = options[:contact_number]
      @delivery_address = options[:delivery_address]
      @items            = options[:items]             || []
      @shipment         = options[:shipment]
      @total            = options[:total]
      @currency         = options[:currency]
      @payment_method   = options[:payment_method]
      @payment_provider = options[:payment_provider]
      @pickup_address   = options[:pickup_address]
      @pickup_at        = options[:pickup_at]
    end

    def self.sample_order
      o = Xpost::Order.new
      o.email = "john@me.com"
      o.buyer_name = "John Doe"
      o.contact_number = "+6391712345678"
      o.shipment = "small-pouch"
      o.total = 1000
      o.currency = "PHP"
      o.payment_method = "cod"
      o.payment_provider = "lbcx"
      o.add_item(Xpost::Item.new(
        type: "product",
        description: "red shirt",
        amount: 1000,
        quantity: 1,
        metadata: "Small"
      ))
      o.delivery_address = Xpost::Address.new(
        name: "Bobby Axelrod",
        line_1: "BGC Corporate Centre",
        line_2: "5th floor",
        city: "Taguig",
        state: "Metro Manila",
        postal_code: "1634",
        country: "PH",
        company: "Axe Capital",
        title: "CEO",
        email: "bobby@axe.com",
        phone_number: "12345678",
        mobile_number: "091712345678",
        fax_number: "12345678",
        district: "BGC",
        remarks: "Send ASAP"
      )
      o
    end

    def confirmable?
      self.valid? && self.has_delivery_address? && self.has_payment_method? && self.has_items?
    end

    # https://docs.quadx.xyz/#5f38afd0-66bb-468f-ab4e-993d4745a257
    # Sets the order status to canceled. An order can be canceled provided they have not passed the cutoff date and time as specified in the contract.
    def cancellable?
      true
    end

    # https://docs.quadx.xyz/#c831dc25-2de4-36c5-c321-8e9db0fc2105
    # The following fields can be updated provided they have not passed the cutoff date and time as specified in the contract.
    def updatable?
      true
    end

    def has_delivery_address?
      self.delivery_address.present?
    end

    def has_payment_method?
      self.total.present? && self.currency.present? && self.payment_method.present? && self.payment_provider.present?
    end

    def has_items?
      self.items.present? ? self.items.count > 0 : false
    end

    def add_item(item)
      self.items << item if item.valid?
    end

    def order_items
      items = []
      self.items.each do |item|
        items << item.attributes
      end
      items
    end

    def attributes
      {
        reference_id: self.reference_id,
        email: self.email,
        buyer_name: self.buyer_name,
        contact_number: self.contact_number,
        shipment: self.shipment,
        total: self.total,
        currency: self.currency,
        payment_method: self.payment_method,
        payment_provider: self.payment_provider,
        items: self.order_items,
        delivery_address: self.delivery_address.attributes,
        pickup_address: self.pickup_address.nil? ? nil : self.pickup_address.attributes,
        pickup_at: self.pickup_at
      }
    end

    def to_json
      self.attributes.to_json
    end
  end
end