require 'json'
require 'set'
require 'typhoeus'

# Ideal Order.create flow
#   order = Order.create({...})
#   order.confirm / order.cancel
#   order.for_pickup(pickup_address, pickup_date)

module Xpost
  class Orders
    # cant actually get all orders since its paginated
    def self.all(auth_token, status = nil)
      valid_xpost_statuses = ["pending", "confirmed", "for_pickup", "picked_up",
                              "failed_pickup", "failed_delivery", "claimed",
                              "return_in_transit", "returned", "failed_return",
                              "out_for_delivery", "for_consolidation", "consolidated",
                              "for_dropoff", "dropped_off", "for_transfer", "for_acceptance",
                              "accepted", "canceled", "in_transit", "delivered"]

      if valid_xpost_statuses.include?(status)
        all_orders_url = "orders?status=#{status}"
      else
        all_orders_url = "orders"
      end

      get_url(all_orders_url, headers: auth_header(auth_token))
    end

    # gets all the details of an order
    def self.find(auth_token, tracking_number)
      find_order_url  = "orders/#{tracking_number}"

      res = get_url(find_order_url, headers: auth_header(auth_token))
      res = res[:message] == "Not Found" ? "Not Found" : res
    end

    # only gets the order tat (turnaround time) info
    def self.track(auth_token, tracking_number)
      track_order_url = "orders/#{tracking_number}"

      res = get_url(track_order_url, headers: auth_header(auth_token))
      res = res[:message] == "Not Found" ? "Not Found" : { timeline: res[:tat], events: res[:events] }
    end

    def self.exists?(auth_token, tracking_number)
      find_order_url  = "orders/#{tracking_number}"
      res = get_url(find_order_url, headers: auth_header(auth_token))
      res[:status] != 404 && res[:message] != "Not Found"
    end

    def self.create(auth_token, order = nil, shipment = nil, reference_id = nil)
      raise "Missing order data" if order.nil?
      raise "Missing shipment data" if shipment.nil?

      order.shipment = shipment
      order.reference_id = reference_id

      res = post_url("orders", data: order, headers: auth_header(auth_token))
    end

    def self.confirm(auth_token, tracking_number, payment_reference_id = nil)
      # TODO: these checks are expensive (potentially three back and forth requests)
      raise "Order not found" unless Xpost::Orders.exists?(auth_token, tracking_number)
      raise "Order isn't pending" if Xpost::Orders.find(auth_token, tracking_number)[:status] != "pending"

      confirm_order_url = "orders/#{tracking_number}/confirm"

      res = put_url(confirm_order_url, data: payment_reference_id, headers: auth_header(auth_token))
      res = res[:message] == "Not Found" ? "Not Found" : res
    end

    def self.cancel(auth_token, tracking_number)
      cancel_order_url = "orders/#{tracking_number}/cancel"
      raise "Order isn't pending" if Xpost::Orders.find(auth_token, tracking_number)[:status] != "pending"

      res = put_url(cancel_order_url, headers: auth_header(auth_token))
      res = res[:message] == "Not Found" ? "Not Found" : res
    end

    def self.for_pickup(auth_token, tracking_number, pickup_address)
      raise "Order isn't confirmed" if Xpost::Orders.find(auth_token, tracking_number)[:status] != "confirmed"
      for_pickup_url = "orders/#{tracking_number}/for-pickup"

      raise "Missing pickup address" if pickup_address.nil?
      raise "Invalid pickup address" unless pickup_address.valid?
      pickup_address = { pickup_address: pickup_address.attributes }

      res = put_url(for_pickup_url, data: pickup_address, headers: auth_header(auth_token))
      res = res[:message] == "Not Found" ? "Not Found" : res
    end

    def self.get_shipping_labels(auth_token, tracking_number)
      if tracking_number.kind_of?(Array)
        shipping_labels_url = "orders/download/awb/zebra"

        res = post_url(shipping_labels_url, data: tracking_number, headers: auth_header(auth_token))
        res = res[:message] == "Not Found" ? "Not Found" : res
      else
        raise "Order isn't for_pickup" if Xpost::Orders.find(auth_token, tracking_number)[:status] != "for_pickup"
        shipping_label_url = "orders/#{tracking_number}/awb?type=zebra"

        res = get_url(shipping_label_url, headers: auth_header(auth_token))
        res = res[:message] == "Not Found" ? "Not Found" : res
      end
    end

    private

    def self.base_url
      Xpost.configuration.production_url
    end

    def self.auth_header(auth_token)
      {"Authorization" => "Bearer #{auth_token}"}
    end

    def self.get_url(url, options = {})
      req_url = "#{base_url}/#{url}"
      req_options = {headers: options[:headers]}

      res = Typhoeus::Request.get(req_url, req_options).body
      res = JSON.parse(res, symbolize_names: true)
    end

    def self.post_url(url, options = {})
      headers     = { 'Content-Type': 'application/json' }.merge options[:headers]
      req_url     = "#{base_url}/#{url}"
      req_options = { headers: headers, body: options[:data].to_json }

      res = Typhoeus::Request.post(req_url, req_options).body
      res = JSON.parse(res, symbolize_names: true)
    end

    def self.put_url(url, options = {})
      headers     = { 'Content-Type': 'application/json' }.merge options[:headers]
      req_url     = "#{base_url}/#{url}"
      req_options = { headers: headers, body: options[:data].to_json }

      res = Typhoeus::Request.put(req_url, req_options).body
      res = JSON.parse(res, symbolize_names: true)
    end
  end
end